!= gtvaropen.f90 - gtool4 ǡΥץ
!
! Authors::   Yasuhiro MORIKAWA, Eizi TOYODA
! Version::   $Id: gtvaropen.f90,v 1.8 2007/08/24 08:01:55 morikawa Exp $
! Tag Name::  $Name: gt4f90io-20070824 $
! Copyright:: Copyright (C) GFD Dennou Club, 2000-2005. All rights reserved.
! License::   See COPYRIGHT[link:../../COPYRIGHT]
!
! ʲΥ֥롼, ؿ gtdata_generic 󶡤ޤ
!

subroutine GTVarOpen(var, url, writable, err)
  !
  !== gtool4 ǡΥץ
  !
  ! *url* Ǽ̤ gtool ѿ򳫤*var* ˳Ǽޤ
  ! *writable*  <tt>.true.</tt> ˻ꤹȽ񤭹߲ĤǳȤޤ
  ! ǥեȤϽ񤭹ԲĤǳޤ
  ! (ޤ *writable* ưݾ㤵Ƥޤ)
  !
  ! Open 줿ѿɬ Close ʤФʤޤ
  !
  ! 顼ȯ硢 *err* Ϳ *err* 
  ! <tt>.true.</tt> Ȥʤä֤ޤ
  !  *err* ͿʤХץߤޤ
  !
  ! *Open*  2 ĤΥ֥롼̾Ǥꡢ
  ! ѿμꤹ뤳ȤǳȤǽǤ
  ! 嵭Υ֥롼򻲾Ȥ
  !
  !=== ­
  ! 
  ! 嵭 Open 򻲾ȤƤ
  !
  use dc_string, only: StrHead
  use dc_types, only: STRING
  use gtdata_types, only: GT_VARIABLE
  use gtdata_generic, only: limit
  use gt_map, only: map_create, vtb_class_netcdf, vtb_class_memory, gtvar_dump
  use an_generic, only: ANVarOpen, an_variable, inquire
  use dc_url, only: url_chop_iorange
  use dc_error, only: StoreError, GT_EFAKE, DC_NOERR, GT_ENOTVAR
  use dc_trace, only: beginsub, endsub
  use dc_present, only: present_and_true
  implicit none
  type(GT_VARIABLE), intent(out):: var
  character(*), intent(in):: url
  logical, intent(in), optional:: writable
  logical, intent(out), optional:: err
  integer:: ndims, stat, cause_i
  character(STRING):: cause_c
  integer, allocatable:: dimlen(:)
  type(an_variable):: an
  character(STRING):: filevar, iorange
  character(*),      parameter:: subname = "GTVarOpen"
  character(*),      parameter:: version = &
    & '$Name: gt4f90io-20070824 $' // &
    & '$Id: gtvaropen.f90,v 1.8 2007/08/24 08:01:55 morikawa Exp $'
continue
  call beginsub(subname, fmt='<%c>', c1=trim(url), version=version)
  stat = DC_NOERR
  cause_i = 0
  cause_c = ''
  var = GT_VARIABLE(-1)
  call url_chop_iorange(url, iorange=iorange, remainder=filevar)
  if (StrHead(filevar, "memory:")) then
    stat = GT_EFAKE
    cause_c = 'GTVarOpen(memory:)'
    goto 999
  else
    call ANVarOpen(an, filevar, writable, err)
    if ( present_and_true(err) ) then
      stat = GT_ENOTVAR
      goto 999
    end if
    call inquire(an, ndims=ndims)
    allocate(dimlen(max(1, ndims)))
    call inquire(an, dimlen=dimlen)
    call map_create(var, vtb_class_netcdf, an%id, ndims, dimlen, stat)
    if (stat /= DC_NOERR) then
      cause_i = ndims
      goto 999
    end if
    deallocate(dimlen)
  endif
  call limit(var, trim(iorange))
  call gtvar_dump(var)
999 continue
  call StoreError(stat, subname, err, cause_c = cause_c, cause_i = cause_i)
  call endsub(subname, 'mapid=%d', i=(/var%mapid/))
end subroutine GTVarOpen
